######################################################################
##### Code for Day 2 of SEM Workshop
##### covering ML fitting of SEMs of observed variables
#####
##### Jarrett E.K. Byrnes
#####
##### Last Modified 2/3/13
######################################################################


#Load your Data File
keeley<-read.csv("./Keeley_rawdata_select4.csv")

#load lavaan
library(lavaan)

##############################
###Intro to Lavaan
##############################

#regression syntax versus lavaan
aLM<-lm(cover ~ age, data=keeley)

aSEM<-sem('cover ~ age', data=keeley)

#output from both
summary(aSEM)

summary(aLM)

#we want out intercept!
aMeanSEM<-sem('cover ~ age', data=keeley, meanstructure=T)

summary(aMeanSEM)


#standardized coefficients
standardizedSolution(aSEM)

summary(aSEM, standardized=T, rsq=T)

########################
# SEM with multiple paths
########################

partialMedModel<-' firesev ~ age
                cover ~ firesev + age'

partialMedSEM<-sem(partialMedModel, 
                   data=keeley)

#look at coefficients
summary(partialMedSEM, rsquare=T, standardized=T)

residuals(partialMedSEM, type="cor")

#plot
library(semPlot)
semPaths(partialMedSEM, "std")

#what if we know better
zeroMedModel<-'firesev ~ 0*age
              cover ~ 0*firesev + age'

zeroMedFit<-sem(zeroMedModel, data=keeley)
summary(zeroMedFit, rsquare=T)
standardizedSolution(zeroMedFit)

#what about correlations
corModel <-'firesev ~ age
            cover ~ age
            cover ~~ firesev'

corFit <- sem(corModel, data=keeley)
standardizedSolution(corFit)

##############
#Morning Exercise
##############

#The Richness Partial Mediation Model
distModel <- 'rich ~ distance + abiotic + hetero
              hetero ~ distance
              abiotic ~ distance'

distFit <- sem(distModel, data=keeley)

summary(distFit, std=T, rsquare=T)
standardizedSolution(distFit)

oneDistModel <- 'rich ~ distance + abiotic + 1*hetero
              hetero ~ distance
              abiotic ~ distance'
oneFit<-sem(oneDistModel, data=keeley)
summary(oneFit, std=T, rsquare=T)


totDistModel <- 'rich ~ a*distance + b*abiotic + c*hetero
              hetero ~ b1*distance
              abiotic ~ c1*distance
              
              direct:=  a
              indirect:= b1*b + c1*c 
              total:= b1*b + c1*c + a
              '
totDistFit<-sem(totDistModel, data=keeley)
summary(totDistFit, std=T, rsquare=T)

standardizedSolution(totDistFit)

###############
# Evaluating Fit
###############

fullMedModel<-' firesev ~ age
                cover ~ firesev'

fullMedSEM<-sem(fullMedModel, 
                data=keeley)

#get fit statistics.  All of 'em
summary(fullMedSEM)
summary(fullMedSEM, fit.measures=T)


#Residuals
residuals(fullMedSEM)

#correlation matrix
residuals(fullMedSEM, type="cor")

#obs v. predicted
inspect(fullMedSEM, "sample")
fitted(fullMedSEM)

#Modification Indices
modificationIndices(fullMedSEM, standardized=F)


####################################
# Mispecification Exercise
####################################

#Full Mediation
distModel2 <- 'rich ~ abiotic  + hetero
              hetero ~ distance
              abiotic ~ distance'

distFit2 <- sem(distModel2, data=keeley)

#see fit
summary(distFit2)
inspect(distFit2, "fit.indices")

#residual correlation
residuals(distFit2, type="cor")

#modification indices, with a trick to only see big ones
modI<-modificationIndices(distFit2, standardized=F)
modI[which(modI$mi>3),]

####################################
# Non-normality 
####################################

#SB Chisq
distFit2SB<-sem(distModel2, data=keeley, estimator="MLM")
summary(distFit2SB)

#Bootstrap
distFit2Boot<-sem(distModel2, data=keeley, 
                  test="bollen.stine", se="boot", bootstrap=100)

summary(distFit2Boot)

distFitBoot<-sem(distModel, data=keeley, 
                  test="bollen.stine", se="boot", bootstrap=100)
summary(distFitBoot)


####################################
# Testing Mediation
####################################

#with a LRT
anova(partialMedSEM, fullMedSEM)

#with aicc
source("./lavaan.modavg.R")
aictab.lavaan(list(fullMedSEM, partialMedSEM), 
                c("Full", "Partial"))

#Exercise
anova(distFit, distFit2)
aictab.lavaan(list(distFit2, distFit), 
              c("Full", "Partial"))



##############################################################################################################################
# ANOVA, ANCOVA, and SEM
##############################################################################################################################

#use getCov to turn lower covariance matrix into a full covariance matrix
lower<-"0.24,    			
-0.104,	3.426,				
-0.0085,	-0.163,	0.412,			
-0.207,	0.871,	0.062,	0.893,		
-0.527,	1.203,	0.098,	0.766,	1.92,	
0.239,	-0.312,	0.103,	-0.35,	-0.651,	0.466"

whalenCov <- getCov(lower, names=c("pesticide",  "macroalgae",	"grass",	"LNCaprell",	"LNGamm",	"lnchla"))
whalenMeans <-c(0.4,  0.702,  1.374,	1.044,	2.374,	-0.254)
whalenN<-40

#ANOVA
anovaModel<-'lnchla ~ pesticide'
anovaFit<-sem(anovaModel, sample.cov=whalenCov, sample.mean=whalenMeans, sample.nobs=whalenN)

summary(anovaFit, standardized=T, rsq=T)

#what about ANCOVA
#note what I'm doing to eliminate the covariances with pesticides
ancovaModel <- 'lnchla ~ pesticide + macroalgae + grass
                pesticide ~~ 0*macroalgae + 0*grass'

ancovaModel2 <- 'lnchla ~ pesticide + macroalgae + grass'

ancovaFitNoFix <- sem(ancovaModel, sample.cov=whalenCov, sample.mean=whalenMeans, sample.nobs=whalenN, fixed.x=F)
ancovaFitFixed <- sem(ancovaModel, sample.cov=whalenCov, sample.mean=whalenMeans, sample.nobs=whalenN, fixed.x=T)
ancovaFitNormal <- sem(ancovaModel2, sample.cov=whalenCov, sample.mean=whalenMeans, sample.nobs=whalenN)

#show effects of fixing x
fitMeasures(ancovaFit)[2]
fitMeasures(ancovaFitFixed)[2]

summary(ancovaFit, standardized=T)
standardizedSolution(ancovaFit)

fullModel <- 'lnchla ~ macroalgae + grass + LNGamm
              LNGamm ~ macroalgae + grass + pesticide
                pesticide ~~ 0*macroalgae + 0*grass'

partialModel <- 'lnchla ~ macroalgae + grass + LNGamm + pesticide
                LNGamm ~ macroalgae + grass + pesticide
                pesticide ~~ 0*macroalgae + 0*grass'

fullFit<-sem(fullModel, sample.cov=whalenCov, sample.mean=whalenMeans, sample.nobs=whalenN, fixed.x=F)
partialFit<-sem(partialModel, sample.cov=whalenCov, sample.mean=whalenMeans, sample.nobs=whalenN, fixed.x=F)

#comparison
anova(fullFit, partialFit)
aictab.lavaan(list(fullFit, partialFit), c("full", "partial"))
aictab.lavaan(list(fullFit, partialFit), c("full", "partial"), second.ord=F)


#the ultimate model
finalModel <- 'lnchla ~ macroalgae + grass + LNGamm + LNCaprell
               LNCaprell ~ macroalgae + grass + pesticide
               LNGamm ~ macroalgae + grass + pesticide
               pesticide ~~ 0*macroalgae + 0*grass'

finalFit<-sem(finalModel, sample.cov=whalenCov, sample.mean=whalenMeans, sample.nobs=whalenN, fixed.x=F)
summary(finalFit, standardized=T, rsq=T)



##############################################################################################################################
# Multiple Categorical Predictors
##############################################################################################################################

urchinData<-read.csv("./urchin_ex_sem.csv")

head(urchinData)
summary(urchinData)

#Make treatment into a series of binary variables
source("./makeBinaryTreatments.R")

binTrt<-makeBinaryTreatments(urchinData, "treatment")
head(binTrt)

#look at bad properties
cor(binTrt)
solve(cor(binTrt))

#add new columns to data frame
urchinData<-cbind(urchinData, binTrt)

urchinModel<-' 
Feeding.rate.dry ~ MAPY + R
GONAD_INDEX ~ MAPY + R + Feeding.rate.dry
'

urchinSEM<-sem(urchinModel, data=urchinData)

summary(urchinSEM, rsquare=T, standardized=T)

##################
####Exercise
##################


urchinModel2<-' 
Feeding.rate.dry ~ MAPY + R
GONAD_INDEX ~ MAPY + R
'
urchinSEM2<-sem(urchinModel2, data=urchinData)

urchinSEM2<-lavaan(urchinModel2, data=urchinData, auto.cov.y = FALSE, auto.var=TRUE)


urchinModel2a<-' 
Feeding.rate.dry ~ MAPY + R
GONAD_INDEX ~ MAPY + R
GONAD_INDEX ~~ 0*Feeding.rate.dry
'

urchinSEM2a<-sem(urchinModel2a, data=urchinData)

anova(urchinSEM, urchinSEM2a)

aictab.lavaan(list(urchinSEM, urchinSEM2a), c("partial", "full"))


#test change

#will need to divide by 10 to avoid a bad fit
urchinData$TEST_CHANGE2 <- urchinData$TEST_CHANGE/10

testModel<-' 
Feeding.rate.dry ~ MAPY + R
TEST_CHANGE2 ~ MAPY + R + Feeding.rate.dry
'
testSEM<-sem(testModel, data=urchinData)

summary(testSEM)

#######
# Nonlinearities
#######

#read in the data
cards<-read.table("./cardainel_et_al_2009.csv")

#make a new nonlinear column
cards$logN2 <- cards$logN^2

cardModel<-'
  SA ~ logN + logN2 + SR
logChl ~ SA + logN
SR ~~ 0*logN + 0*logN2 
logN ~~ logN2
'

#fixed.x=F in order to allow the dropping of paths between
#SR and logN and LogNcen2
cardFit <- sem(cardModel, data=cards, fixed.x=F)

summary(cardFit, standardized=T, rsquare=T)


#make a new nonlinear column
cards$logN2Cen <- (cards$logN-mean(cards$logN))^2

cardModel2<-'
  SA ~ logN + logN2Cen + SR
logChl ~ SA + logN
SR ~~ 0*logN + 0*logN2Cen 
logN ~~ logN2Cen
'

cardFit2 <- sem(cardModel2, data=cards, fixed.x=F)

summary(cardFit2, standardized=T, rsquare=T)